@@ -1,5 +1,5 @@
 # This module is a lite version of LinAlg.py module which contains
-# high-level Python interface to the LAPACK library.  The lite versioho
+# high-level Python interface to the LAPACK library.  The lite version
 # only accesses the following LAPACK functions: dgesv, zgesv, dgeev,
 # zgeev, dgesdd, zgesdd, dgelsd, zgelsd, dsyevd, zheevd, dgetrf, dpotrf.
 
@@ -11,7 +11,8 @@ import MLab
 import multiarray
 
 # Error object
-LinAlgError = 'LinearAlgebraError'
+class LinAlgError(Exception):
+    pass
 
 # Helper routines
 _lapack_type = {'f': 0, 'd': 1, 'F': 2, 'D': 3}
@@ -70,10 +71,10 @@ def _assertSquareness(*arrays):
     for a in arrays:
         if max(a.shape) != min(a.shape):
             raise LinAlgError, 'Array must be square'
-    
+
 
 # Linear equations
-        
+
 def solve_linear_equations(a, b):
     one_eq = len(b.shape) == 1
     if one_eq:
@@ -91,7 +92,7 @@ def solve_linear_equations(a, b):
     else:
         lapack_routine = lapack_lite.dgesv
     a, b = _fastCopyAndTranspose(t, a, b)
-    pivots = Numeric.zeros(n_eq, 'l')
+    pivots = Numeric.zeros(n_eq, 'i')
     results = lapack_routine(n_eq, n_rhs, a, n_eq, pivots, b, n_eq, 0)
     if results['info'] > 0:
         raise LinAlgError, 'Singular matrix'
@@ -116,9 +117,9 @@ def cholesky_decomposition(a):
     m = a.shape[0]
     n = a.shape[1]
     if _array_kind[t] == 1:
-	lapack_routine = lapack_lite.zpotrf
+        lapack_routine = lapack_lite.zpotrf
     else:
-	lapack_routine = lapack_lite.dpotrf
+        lapack_routine = lapack_lite.dpotrf
     results = lapack_routine('L', n, a, m, 0)
     if results['info'] > 0:
         raise LinAlgError, 'Matrix is not positive definite - Cholesky decomposition cannot be computed'
@@ -175,8 +176,8 @@ def Heigenvalues(a, UPLO='L'):
     real_t = _array_type[0][_array_precision[t]]
     a = _castCopyAndTranspose(t, a)
     n = a.shape[0]
-    liwork = 5*n+3 
-    iwork = Numeric.zeros((liwork,),'l')
+    liwork = 5*n+3
+    iwork = Numeric.zeros((liwork,),'i')
     if _array_kind[t] == 1: # Complex routines take different arguments
         lapack_routine = lapack_lite.zheevd
         w = Numeric.zeros((n,), real_t)
@@ -207,7 +208,7 @@ def Heigenvalues(a, UPLO='L'):
 
 def eigenvectors(a):
     """eigenvectors(a) returns u,v  where u is the eigenvalues and
-v is a matrix of eigenvectors with vector v[i] corresponds to 
+v is a matrix of eigenvectors with vector v[i] corresponds to
 eigenvalue u[i].  Satisfies the equation dot(a, v[i]) = u[i]*v[i]
 """
     _assertRank2(a)
@@ -270,7 +271,7 @@ def Heigenvectors(a, UPLO='L'):
     a = _castCopyAndTranspose(t, a)
     n = a.shape[0]
     liwork = 5*n+3
-    iwork = Numeric.zeros((liwork,),'l')
+    iwork = Numeric.zeros((liwork,),'i')
     if _array_kind[t] == 1: # Complex routines take different arguments
         lapack_routine = lapack_lite.zheevd
         w = Numeric.zeros((n,), real_t)
@@ -297,7 +298,7 @@ def Heigenvectors(a, UPLO='L'):
         raise LinAlgError, 'Eigenvalues did not converge'
     return (w,a)
 
-   
+
 # Singular value decomposition
 
 def singular_value_decomposition(a, full_matrices = 0):
@@ -308,17 +309,17 @@ def singular_value_decomposition(a, full_matrices = 0):
     real_t = _array_type[0][_array_precision[t]]
     a = _fastCopyAndTranspose(t, a)
     if full_matrices:
-	nu = m
-	nvt = n
-	option = 'A'
+        nu = m
+        nvt = n
+        option = 'A'
     else:
-	nu = min(n,m)
-	nvt = min(n,m)
-	option = 'S'
+        nu = min(n,m)
+        nvt = min(n,m)
+        option = 'S'
     s = Numeric.zeros((min(n,m),), real_t)
     u = Numeric.zeros((nu, m), t)
     vt = Numeric.zeros((n, nvt), t)
-    iwork = Numeric.zeros((8*min(m,n),), 'l')
+    iwork = Numeric.zeros((8*min(m,n),), 'i')
     if _array_kind[t] == 1: # Complex routines take different arguments
         lapack_routine = lapack_lite.zgesdd
         rwork = Numeric.zeros((5*min(m,n)*min(m,n) + 5*min(m,n),), real_t)
@@ -348,6 +349,9 @@ def singular_value_decomposition(a, full_matrices = 0):
 # Generalized inverse
 
 def generalized_inverse(a, rcond = 1.e-10):
+    a = Numeric.array(a, copy=0)
+    if a.typecode() in Numeric.typecodes['Complex']:
+        a = Numeric.conjugate(a)
     u, s, vt = singular_value_decomposition(a, 0)
     m = u.shape[0]
     n = vt.shape[1]
@@ -355,10 +359,10 @@ def generalized_inverse(a, rcond = 1.e-10):
     for i in range(min(n,m)):
         if s[i] > cutoff:
             s[i] = 1./s[i]
-	else:
-	    s[i] = 0.;
+        else:
+            s[i] = 0.;
     return Numeric.dot(Numeric.transpose(vt),
-		       s[:, Numeric.NewAxis]*Numeric.transpose(u))
+                       s[:, Numeric.NewAxis]*Numeric.transpose(u))
 
 # Determinant
 
@@ -369,27 +373,27 @@ def determinant(a):
     a = _fastCopyAndTranspose(t, a)
     n = a.shape[0]
     if _array_kind[t] == 1:
-	lapack_routine = lapack_lite.zgetrf
+        lapack_routine = lapack_lite.zgetrf
     else:
-	lapack_routine = lapack_lite.dgetrf
-    pivots = Numeric.zeros((n,), 'l')
+        lapack_routine = lapack_lite.dgetrf
+    pivots = Numeric.zeros((n,), 'i')
     results = lapack_routine(n, n, a, n, pivots, 0)
     sign = Numeric.add.reduce(Numeric.not_equal(pivots,
-						Numeric.arrayrange(1, n+1))) % 2
+                                                Numeric.arrayrange(1, n+1))) % 2
     return (1.-2.*sign)*Numeric.multiply.reduce(Numeric.diagonal(a))
 
-# Linear Least Squares 
-        
+# Linear Least Squares
+
 def linear_least_squares(a, b, rcond=1.e-10):
-    """solveLinearLeastSquares(a,b) returns x,resids,rank,s 
-where x minimizes 2-norm(|b - Ax|) 
+    """solveLinearLeastSquares(a,b) returns x,resids,rank,s
+where x minimizes 2-norm(|b - Ax|)
       resids is the sum square residuals
       rank is the rank of A
-      s is an rank of the singual values of A in desending order
+      s is the rank of the singular values of A in descending order
 
 If b is a matrix then x is also a matrix with corresponding columns.
 If the rank of A is less than the number of columns of A or greater than
-the numer of rows, then residuals will be returned as an empty array
+the number of rows, then residuals will be returned as an empty array
 otherwise resids = sum((b-dot(A,x)**2).
 Singular values less than s[0]*rcond are treated as zero.
 """
@@ -410,7 +414,7 @@ Singular values less than s[0]*rcond are treated as zero.
     a,bstar = _castCopyAndTranspose(t, a, bstar)
     s = Numeric.zeros((min(m,n),),real_t)
     nlvl = max( 0, int( math.log( float(min( m,n ))/2. ) ) + 1 )
-    iwork = Numeric.zeros((3*min(m,n)*nlvl+11*min(m,n),), 'l')
+    iwork = Numeric.zeros((3*min(m,n)*nlvl+11*min(m,n),), 'i')
     if _array_kind[t] == 1: # Complex routines take different arguments
         lapack_routine = lapack_lite.zgelsd
         lwork = 1
@@ -450,62 +454,61 @@ Singular values less than s[0]*rcond are treated as zero.
         x = copy.copy(Numeric.transpose(bstar)[:n,:])
         if (results['rank']==n) and (m>n):
             resids = copy.copy(Numeric.sum((Numeric.transpose(bstar)[n:,:])**2))
-    return x,resids,results['rank'],copy.copy(s[:min(n,m)]) 
+    return x,resids,results['rank'],copy.copy(s[:min(n,m)])
 
 
 if __name__ == '__main__':
-	from Numeric import *
+    from Numeric import *
 
-	def test(a, b):
+    def test(a, b):
 
-		print "All numbers printed should be (almost) zero:"
+        print "All numbers printed should be (almost) zero:"
 
-		x = solve_linear_equations(a, b)
-		check = b - matrixmultiply(a, x)
-		print check
+        x = solve_linear_equations(a, b)
+        check = b - matrixmultiply(a, x)
+        print check
 
 
-		a_inv = inverse(a)
-		check = matrixmultiply(a, a_inv)-identity(a.shape[0])
-		print check
+        a_inv = inverse(a)
+        check = matrixmultiply(a, a_inv)-identity(a.shape[0])
+        print check
 
 
-		ev = eigenvalues(a)
+        ev = eigenvalues(a)
 
-		evalues, evectors = eigenvectors(a)
-		check = ev-evalues
-		print check
+        evalues, evectors = eigenvectors(a)
+        check = ev-evalues
+        print check
 
-		evectors = transpose(evectors)
-		check = matrixmultiply(a, evectors)-evectors*evalues
-		print check
+        evectors = transpose(evectors)
+        check = matrixmultiply(a, evectors)-evectors*evalues
+        print check
 
 
-		u, s, vt = singular_value_decomposition(a)
-		check = a - Numeric.matrixmultiply(u*s, vt)
-		print check
+        u, s, vt = singular_value_decomposition(a)
+        check = a - Numeric.matrixmultiply(u*s, vt)
+        print check
 
 
-		a_ginv = generalized_inverse(a)
-		check = matrixmultiply(a, a_ginv)-identity(a.shape[0])
-		print check
+        a_ginv = generalized_inverse(a)
+        check = matrixmultiply(a, a_ginv)-identity(a.shape[0])
+        print check
 
 
-		det = determinant(a)
-		check = det-multiply.reduce(evalues)
-		print check
+        det = determinant(a)
+        check = det-multiply.reduce(evalues)
+        print check
 
-		x, residuals, rank, sv = linear_least_squares(a, b)
-		check = b - matrixmultiply(a, x)
-		print check
-		print rank-a.shape[0]
-		print sv-s
+        x, residuals, rank, sv = linear_least_squares(a, b)
+        check = b - matrixmultiply(a, x)
+        print check
+        print rank-a.shape[0]
+        print sv-s
 
-	a = array([[1.,2.], [3.,4.]])
-	b = array([2., 1.])
-	test(a, b)
-
-	a = a+0j
-	b = b+0j
-	test(a, b)
+    a = array([[1.,2.], [3.,4.]])
+    b = array([2., 1.])
+    test(a, b)
 
+    a = a+0j
+    b = b+0j
+    test(a, b)
