@@ -5,7 +5,7 @@ Functions
 
 -   array                      - NumPy Array construction
 -   zeros                      - Return an array of all zeros
--   empty                      - Return an unitialized array (200x faster than zeros)
+-   empty                      - Return an uninitialized array (200x faster than zeros)
 -   shape                      - Return shape of sequence or array
 -   rank                       - Return number of dimensions
 -   size                       - Return number of elements in entire array or a
@@ -13,7 +13,7 @@ Functions
 -   fromstring                 - Construct array from (byte) string
 -   take                       - Select sub-arrays using sequence of indices
 -   put                        - Set sub-arrays using sequence of 1-D indices
--   putmask                    - Set portion of arrays using a mask 
+-   putmask                    - Set portion of arrays using a mask
 -   reshape                    - Return array with new shape
 -   repeat                     - Repeat elements of array
 -   choose                     - Construct new array from indexed array tuple
@@ -32,14 +32,14 @@ More Functions:
 
 -   arrayrange (arange)        - Return regularly spaced array
 -   asarray                    - Guarantee NumPy array
--   sarray                     - Guarantee a NumPy array that keeps precision 
+-   sarray                     - Guarantee a NumPy array that keeps precision
 -   convolve                   - Convolve two 1-d arrays
 -   swapaxes                   - Exchange axes
 -   concatenate                - Join arrays together
 -   transpose                  - Permute axes
 -   sort                       - Sort elements of array
 -   argsort                    - Indices of sorted array
--   argmax                     - Index of largest value                      
+-   argmax                     - Index of largest value
 -   argmin                     - Index of smallest value
 -   innerproduct               - Innerproduct of two arrays
 -   dot                        - Dot product (matrix multiplication)
@@ -53,7 +53,7 @@ More Functions:
 -   dumps                      - Return pickled string representing data
 -   load                       - Return array stored in file object
 -   loads                      - Return array from pickled string
--   ravel                      - Return array as 1-D 
+-   ravel                      - Return array as 1-D
 -   nonzero                    - Indices of nonzero elements for 1-D array
 -   shape                      - Shape of array
 -   where                      - Construct array from binary result
@@ -62,34 +62,34 @@ More Functions:
 -   ones                       - Array of all ones
 -   identity                   - 2-D identity array (matrix)
 
-(Universal) Math Functions 
+(Universal) Math Functions
 
-       add                    logical_or             exp        
-       subtract               logical_xor            log        
-       multiply               logical_not            log10      
-       divide                 maximum                sin        
-       divide_safe            minimum                sinh       
-       conjugate              bitwise_and            sqrt       
-       power                  bitwise_or             tan        
-       absolute               bitwise_xor            tanh       
-       negative               invert                 ceil       
-       greater                left_shift             fabs       
-       greater_equal          right_shift            floor      
-       less                   arccos                 arctan2    
-       less_equal             arcsin                 fmod       
-       equal                  arctan                 hypot      
-       not_equal              cos                    around     
+       add                    logical_or             exp
+       subtract               logical_xor            log
+       multiply               logical_not            log10
+       divide                 maximum                sin
+       divide_safe            minimum                sinh
+       conjugate              bitwise_and            sqrt
+       power                  bitwise_or             tan
+       absolute               bitwise_xor            tanh
+       negative               invert                 ceil
+       greater                left_shift             fabs
+       greater_equal          right_shift            floor
+       less                   arccos                 arctan2
+       less_equal             arcsin                 fmod
+       equal                  arctan                 hypot
+       not_equal              cos                    around
        logical_and            cosh                   sign
        arccosh                arcsinh                arctanh
 
-"""   
+"""
 
 import numeric_version
 __version__ = numeric_version.version
 del numeric_version
 
 import multiarray
-from umath import * 
+from umath import *
 from Precision import *
 
 import _numpy # for freeze dependency resolution (at least on Mac)
@@ -103,10 +103,10 @@ NewAxis = None
 #in C some day
 
 ##def range2(start, stop=None, step=1, typecode=None):
-##    """Just like range() except it returns a array whose type can be specfied
+##    """Just like range() except it returns a array whose type can be specified
 ##    by the keyword argument typecode.
 ##    """
-##    
+##
 ##    if (stop is None):
 ##        stop = start
 ##        start = 0
@@ -164,12 +164,12 @@ def put (a, ind, v):
     """put(a, ind, v) results in a[n] = v[n] for all n in ind
        If v is shorter than mask it will be repeated as necessary.
        In particular v can be a scalar or length 1 array.
-       The routine put is the equivalent of the following (although the loop 
-       is in C for speed): 
+       The routine put is the equivalent of the following (although the loop
+       is in C for speed):
 
-           ind = array(indices, copy=0) 
-           v = array(values, copy=0).astype(a, typecode()) 
-           for i in ind: a.flat[i] = v[i] 
+           ind = array(indices, copy=0)
+           v = array(values, copy=0).astype(a, typecode())
+           for i in ind: a.flat[i] = v[i]
        a must be a contiguous Numeric array.
     """
     multiarray.put (a, ind, array(v, copy=0).astype(a.typecode()))
@@ -278,7 +278,7 @@ def argsort(a, axis=-1):
 
 def argmax(a, axis=-1):
     """argmax(a,axis=-1) returns the indices to the maximum value of the
-    1-D arrays along the given axis.    
+    1-D arrays along the given axis.
     """
     a = array(a, copy=0)
     n = len(a.shape)
@@ -292,7 +292,7 @@ def argmax(a, axis=-1):
 
 def argmin(a, axis=-1):
     """argmin(a,axis=-1) returns the indices to the minimum value of the
-    1-D arrays along the given axis.    
+    1-D arrays along the given axis.
     """
     arra = array(a,copy=0)
     type = arra.typecode()
@@ -303,7 +303,7 @@ def argmin(a, axis=-1):
     n = len(a.shape)
     if axis < 0: axis += n
     if axis < 0 or axis >= n:
-        raise ValueError, "argmin axis argument out of bounds" 
+        raise ValueError, "argmin axis argument out of bounds"
     if axis != n-1: a = swapaxes(a, axis, n-1)
     s = multiarray.argmax(a)
     if axis != n-1: s = swapaxes(s, axis, -1)
@@ -357,10 +357,46 @@ def vdot(a, b):
 try:
     from dotblas import dot, innerproduct, vdot
 except ImportError: pass
-        
+
 #This is obsolete, don't use in new code
 matrixmultiply = dot
 
+def _move_axis_to_0(a, axis):
+    if axis == 0:
+        return a
+    n = len(a.shape)
+    if axis < 0:
+        axis += n
+    axes = range(1, axis+1) + [0,] + range(axis+1, n)
+    return multiarray.transpose(a, axes)
+
+def cross_product(a, b, axis1=-1, axis2=-1):
+    """Return the cross product of two vectors.
+
+    The cross product is performed over the last axes of a and b by default,
+    and can handle axes with dimensions 2 and 3. For a dimension of 2,
+    the z-component of the equivalent three-dimensional cross product is
+    returned.
+    """
+    a = _move_axis_to_0(asarray(a), axis1)
+    b = _move_axis_to_0(asarray(b), axis2)
+    if a.shape[0] != b.shape[0]:
+        raise ValueError("incompatible dimensions for cross product")
+    elif a.shape[0] == 2:
+        return a[0]*b[1] - a[1]*b[0]
+    elif a.shape[0] == 3:
+        x = a[1]*b[2] - a[2]*b[1]
+        y = a[2]*b[0] - a[0]*b[2]
+        z = a[0]*b[1] - a[1]*b[0]
+        cp = array([x,y,z])
+        if len(cp.shape) == 1:
+            return cp
+        # we put the result (in the first axis) as the last axis
+        axes = range(1, len(cp.shape)) + [0]
+        return multiarray.transpose(cp, axes)
+    else:
+        raise ValueError("can only do cross product for axes with dimensions 2 or 3")
+
 #Use Konrad's printing function (modified for both str and repr now)
 from ArrayPrinter import array2string
 def array_repr(a, max_line_width = None, precision = None, suppress_small = None):
@@ -368,7 +404,7 @@ def array_repr(a, max_line_width = None, precision = None, suppress_small = None
 
 def array_str(a, max_line_width = None, precision = None, suppress_small = None):
     return array2string(a, max_line_width, precision, suppress_small, ' ', 0)
-    
+
 multiarray.set_string_function(array_str, 0)
 multiarray.set_string_function(array_repr, 1)
 
@@ -387,7 +423,7 @@ def resize(a, new_shape):
     n_copies = int(total_size / len(a))
     extra = total_size % len(a)
 
-    if extra != 0: 
+    if extra != 0:
         n_copies = n_copies+1
         extra = len(a)-extra
 
@@ -414,19 +450,25 @@ def fromfunction(function, dimensions):
     numbers indicating the length of the desired output for each axis.
     """
     return apply(function, tuple(indices(dimensions)))
-    
+
 
 def diagonal(a, offset= 0, axis1=0, axis2=1):
-    """diagonal(a, offset=0, axis1=0, axis2=1) returns the given diagonals
-    defined by the last two dimensions of the array.
+    """diagonal(a, offset=0, axis1=0, axis2=1) returns all offset diagonals
+    defined by the given dimensions of the array.
     """
-    a = array (a)
-    if axis2 < axis1: axis1, axis2 = axis2, axis1
-    if axis2 > 1:
-        new_axes = range (len (a.shape))
-        del new_axes [axis2]; del new_axes [axis1]
-        new_axes [0:0] = [axis1, axis2]
-        a = transpose (a, new_axes)
+    a = asarray (a)
+    nd = len(a.shape)
+    new_axes = range(nd)
+    if (axis1 < 0): axis1 += nd
+    if (axis2 < 0): axis2 += nd
+    try:
+        new_axes.remove(axis1)
+        new_axes.remove(axis2)
+    except ValueError:
+            raise ValueError, "axis1(=%d) and axis2(=%d) must be different "\
+                  "and within range (nd=%d)." % (axis1, axis2, nd)
+    new_axes = new_axes + [axis1, axis2]
+    a = transpose(a, new_axes)
     s = a.shape
     if len (s) == 2:
         n1 = s [0]
@@ -435,9 +477,11 @@ def diagonal(a, offset= 0, axis1=0, axis2=1):
         s = (n,)
         a = reshape (a, s)
         if offset < 0:
-            return take (a, range ( - n2 * offset, min(n2, n1+offset) * (n2+1) - n2 * offset, n2+1), 0)
+            return take (a, range (-n2*offset, min(n2, n1+offset)*(n2+1) -
+                                   n2*offset, n2+1), 0)
         else:
-            return take (a, range (offset,         min(n1, n2-offset) * (n2+1) + offset,      n2+1), 0)
+            return take (a, range (offset,  min(n1, n2-offset) * (n2+1) +
+                                   offset, n2+1), 0)
     else :
         my_diagonal = []
         for i in range (s [0]) :
@@ -446,18 +490,18 @@ def diagonal(a, offset= 0, axis1=0, axis2=1):
 
 def trace(a, offset=0, axis1=0, axis2=1):
     """trace(a,offset=0, axis1=0, axis2=1) returns the sum along diagonals
-    (defined by the last two dimenions) of the array.
+    (defined by the last two dimensions) of the array.
     """
-    return add.reduce(diagonal(a, offset, axis1, axis2))
+    return add.reduce(diagonal(a, offset, axis1, axis2),-1)
 
 
 # These two functions are used in my modified pickle.py so that
-# matrices can be pickled.  Notice that matrices are written in 
+# matrices can be pickled.  Notice that matrices are written in
 # binary format for efficiency, but that they pay attention to
 # byte-order issues for  portability.
 
-def DumpArray(m, fp):    
-    if m.typecode() == 'O': 
+def DumpArray(m, fp):
+    if m.typecode() == 'O':
         raise TypeError, "Numeric Pickler can't pickle arrays of Objects"
     s = m.shape
     if LittleEndian: endian = "L"
@@ -473,13 +517,13 @@ def LoadArray(fp):
     if ln[0][0] == 'A': ln[0] = ln[0][1:] # Nasty hack showing my ignorance of pickle
     typecode = ln[0][0]
     endian = ln[0][1]
-    
+
     shape = map(lambda x: string.atoi(x), ln[1:])
     itemsize = string.atoi(ln[0][2:])
 
     sz = reduce(multiply, shape)*itemsize
     data = fp.read(sz)
-        
+
     m = fromstring(data, typecode)
     m = reshape(m, shape)
 
@@ -492,61 +536,45 @@ import pickle, copy
 class Unpickler(pickle.Unpickler):
     def load_array(self):
         self.stack.append(LoadArray(self))
-    
-    dispatch = copy.copy(pickle.Unpickler.dispatch)    
+
+    dispatch = copy.copy(pickle.Unpickler.dispatch)
     dispatch['A'] = load_array
 
 class Pickler(pickle.Pickler):
     def save_array(self, object):
         DumpArray(object, self)
 
-    dispatch = copy.copy(pickle.Pickler.dispatch)        
+    dispatch = copy.copy(pickle.Pickler.dispatch)
     dispatch[ArrayType] = save_array
 
 #Convenience functions
 from StringIO import StringIO
 
-def dump(object, file):
-    """dump(object, file) pickles (binary-writes) the object to an open file.
-    """
-    Pickler(file).dump(object)
-
-def dumps(object):
-    """dumps(object) pickles (binary-writes) the object and returns the byte
-    stream.
-    """
-    file = StringIO()
-    Pickler(file).dump(object)
-    return file.getvalue()
-
-def load(file):
-    """load(file) returns an array from the open file pointing to pickled data. 
-    """
-    return Unpickler(file).load()
-
-def loads(str):
-    """loads(str) returns an array from a byte stream containing its pickled
-    representation.
-    """
-    file = StringIO(str)
-    return Unpickler(file).load()
-
-# slightly different format uses the copy_reg mechanism
-import copy_reg
-
-def array_constructor(shape, typecode, thestr, Endian=LittleEndian):
-    x = fromstring(thestr, typecode)
-    x.shape = shape
-    if LittleEndian != Endian:
-        return x.byteswapped()
-    else:
-        return x
-
-def pickle_array(a):
-    return (array_constructor, 
-            (a.shape, a.typecode(), a.tostring(), LittleEndian))
-
-copy_reg.pickle(ArrayType, pickle_array, array_constructor)
+from pickle import load, loads, dump, dumps 
+ 
+# slightly different format uses the copy_reg mechanism 
+import copy_reg 
+ 
+def array_constructor(shape, typecode, thestr, Endian=LittleEndian): 
+    if typecode == "O":
+        x = array(thestr,"O") 
+    else: 
+        x = fromstring(thestr, typecode) 
+    x.shape = shape 
+    if LittleEndian != Endian: 
+        return x.byteswapped() 
+    else: 
+        return x 
+ 
+def pickle_array(a): 
+    if a.typecode() == "O": 
+        return (array_constructor,  
+                (a.shape, a.typecode(), a.tolist(), LittleEndian)) 
+    else: 
+        return (array_constructor,  
+                (a.shape, a.typecode(), a.tostring(), LittleEndian))
+          
+copy_reg.pickle(ArrayType, pickle_array, array_constructor) 
 
 
 # These are all essentially abbreviations
@@ -577,7 +605,7 @@ def where(condition, x, y):
     return choose(not_equal(condition, 0), (y, x))
 
 def compress(condition, m, axis=-1):
-    """compress(condition, x, axis=-1) = those elements of x corresponding 
+    """compress(condition, x, axis=-1) = those elements of x corresponding
     to those elements of condition that are "true".  condition must be the
     same size as the given dimension of x."""
     return take(m, nonzero(condition), axis)
@@ -592,12 +620,12 @@ def clip(m, m_min, m_max):
 
 def ones(shape, typecode='l', savespace=0):
     """ones(shape, typecode=Int, savespace=0) returns an array of the given
-    dimensions which is initialized to all ones. 
+    dimensions which is initialized to all ones.
     """
     a=zeros(shape, typecode, savespace)
     a[...]=1
     return a
- 
+
 def identity(n,typecode='l'):
     """identity(n) returns the identity matrix of shape n x n.
     """
@@ -668,7 +696,7 @@ arange = multiarray.arange
 
 def around(m, decimals=0):
     """around(m, decimals=0) \
-    Round in the same way as standard python performs rounding. Returns 
+    Round in the same way as standard python performs rounding. Returns
     always a float.
     """
     m = asarray(m)
@@ -685,7 +713,7 @@ def around(m, decimals=0):
     else:
         m = m*s
     return m
-    
+
 def sign(m):
     """sign(m) gives an array with shape of m with elements defined by sign
     function:  where m is less than 0 return -1, where m greater than 0, a=1,
@@ -724,19 +752,19 @@ def size (a, axis=None):
     "Get the number of elements in sequence a, or along a certain axis."
     s = shape(a)
     if axis is None:
-        if len(s)==0: 
+        if len(s)==0:
             return 1
-        else: 
+        else:
             return reduce(lambda x,y:x*y, s)
     else:
         return s[axis]
-            
+
 def average (a, axis=0, weights=None, returned = 0):
     """average(a, axis=0, weights=None)
-       Computes average along indicated axis. 
+       Computes average along indicated axis.
        If axis is None, average over the entire array.
        Inputs can be integer or floating types; result is type Float.
-   
+
        If weights are given, result is:
            sum(a*weights)/(sum(weights))
        weights must have a's shape or be the 1-d with length the size
@@ -745,7 +773,7 @@ def average (a, axis=0, weights=None, returned = 0):
        Not supplying weights is equivalent to supply weights that are
        all 1.
 
-       If returned, return a tuple: the result and the sum of the weights 
+       If returned, return a tuple: the result and the sum of the weights
        or count of values. The shape of these two results will be the same.
 
        raises ZeroDivisionError if appropriate when result is scalar.
@@ -757,16 +785,16 @@ def average (a, axis=0, weights=None, returned = 0):
             n = add.reduce(a)
             d = len(a) * 1.0
         else:
-            w = array(weights).flat * 1.0
+            w = array(weights, typecode=Float, copy=0).flat
             n = add.reduce(a*w)
-            d = add.reduce(w) 
+            d = add.reduce(w)
     else:
         a = array(a)
         ash = a.shape
         if ash == ():
             a.shape = (1,)
         if weights is None:
-            n = add.reduce(a, axis) 
+            n = add.reduce(a, axis)
             d = ash[axis] * 1.0
             if returned:
                 d = ones(shape(n)) * d
@@ -777,7 +805,7 @@ def average (a, axis=0, weights=None, returned = 0):
                 wsh = (1,)
             if wsh == ash:
                 n = add.reduce(a*w, axis)
-                d = add.reduce(w, axis) 
+                d = add.reduce(w, axis)
             elif wsh == (ash[axis],):
                 ni = ash[axis]
                 r = [NewAxis]*ni
@@ -787,12 +815,11 @@ def average (a, axis=0, weights=None, returned = 0):
                 d = add.reduce(w1, axis)
             else:
                 raise ValueError, 'average: weights wrong shape.'
-            
+
     if not isinstance(d, ArrayType):
-        if d == 0.0: 
+        if d == 0.0:
             raise ZeroDivisionError, 'Numeric.average, zero denominator'
     if returned:
         return n/d, d
     else:
         return n/d
-
